%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ModelingAndDataExample()
%
% Matlab code for the Bayesian learning model and the Huffman code,
% self-information and probabilistic chunking models from Brady, Konkle,
% Alvarez (under review), "Compression in Visual Working Memory: Using
% Statistical Regularities to Form More Efficient Memory Representations".
%
% TFB - 01/09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    load('subjectData.mat'); % Load example subject data
	
    % Plot the capacity of this subject by block
    for curBlock = 1:10
        pcThisBlock = correctByTrial(block==curBlock);
        pcByBlock(curBlock) = mean(pcThisBlock);
    end
    capacityByBlock = ((pcByBlock*8*8)-8)/7;
    subplot(2,2,1);
    plot(capacityByBlock);
    title('Subject capacity by block');
        
    % Model this subjects data using each of our models
	% - Set alpha to 34, which was the best fit value of the prior on the
	%  learning model across subjects	
	alpha = 34;
	
    modelCodeLength = ModelSubjectCapacity(freqEachPairByBlock, ...
		alpha, @ModelSelfInfo);
    subplot(2,2,2);
    plot(modelCodeLength);
	title('Self information for each color by block');
    
	modelCodeLength = ModelSubjectCapacity(freqEachPairByBlock, ...
		alpha, @ModelHuffmanCode);
    subplot(2,2,3);
	plot(modelCodeLength);
	title('Huffman code length per color by block');
	
	modelCodeLength = ModelSubjectCapacity(freqEachPairByBlock, ...
		alpha, @ModelChunksProbabilistic);
    subplot(2,2,4);
	plot(modelCodeLength);
    title('Chunks needed per color, by block');
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function modelCodeLength=ModelSubjectCapacity(freqEachPairByBlock, ...
    priorForUniform, modelFunc)     
    
    % Setup the modeling structures
    modelCodeLength = zeros(1, 10);
    curEstimateOfProb = zeros(8,8) + priorForUniform;

    for i=1:length(freqEachPairByBlock)       
        % Note: Adding the counts to the prior and then normalizing is
        % equivalent mathematically to a Dirlichet-Multinomial
        % hierarchical Bayes inference. In other words, if the prior is
        % Dirichlet(priorForUniform), then the posterior after seeing some
        % of each of the items is Dirichlet(priorForUniform +
        % freqEachPair). Because the expected value of the Dirichlet is 
        % just the parameters normalized to sum to 1, all we have to do to 
        % get our normalized posterior is normalize this new distribution.
        
        normMat = curEstimateOfProb ./ sum(curEstimateOfProb(:));
		curEstimateOfProb = curEstimateOfProb + freqEachPairByBlock{i};
        modelCodeLength(i) = modelFunc(normMat, freqEachPairByBlock{i});
	end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lenCode = ModelSelfInfo(normMat, freqEachPairThisBlock)
    surpriseBits = -log2(normMat);
    allItems = surpriseBits .* freqEachPairThisBlock;
    lenCode = sum(allItems(:)) ./ sum(freqEachPairThisBlock(:)) ./ 2;
     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lenCode = ModelHuffmanCode(normMat, freqEachPairThisBlock)
    itemCodes = HuffmanCodeSet(normMat(:));
	for i=1:length(itemCodes)
		lengthCodes(i) = length(itemCodes{i});
	end
	allItems = lengthCodes' .* freqEachPairThisBlock(:);
    lenCode = sum(allItems(:)) ./ sum(freqEachPairThisBlock(:)) ./ 2;
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lenCode = ModelChunksProbabilistic(normMat, freqEachPairThisBlock)
	freeP = 15; % 15 is the free parameter on the chunking model that 
				% provides the best fit
    lenCode = 0;
    for q=1:length(freqEachPairThisBlock(:))
		for i=1:freqEachPairThisBlock(q)
			if rand < (normMat(q)*freeP) 
				lenCode = lenCode + 1;
			else
				lenCode = lenCode + 2;
			end
		end
    end
    lenCode  = lenCode ./ sum(freqEachPairThisBlock(:));

	
      
	
	